var EventHelper = (function() {
    // An array to keep track of registered event listeners.
    var eventListeners = [];

    // Removes the listener from the node.
    function detachListener(event) {
        var nodeObj = event.nodeObj;

        if (nodeObj.removeEventListener) {
            nodeObj.removeEventListener(event.name, event.listener, false);
        } else if (nodeObj.detachEvent) {
            nodeObj.detachEvent("on" + event.name, event.listener);
        }
    };

    // Removes the listener from the eventListeners array.
    function deleteListener(event) {
        var tempArray = [];

        for (var i = 0; i < eventListeners.length; i++) {
            if (eventListeners[i] != event) {
                tempArray.push(eventListeners[i]);
            }
        }

        eventListeners = tempArray;
    };

    // Removes all listeners to prevent memory leaks in IE.
    function unbindAllListeners() {
        for (var i = 0; i < eventListeners.length; i++) {
            detachListener(eventListeners[i]);
        }
        eventListeners.length = 0;
    };

    var helper = {
        // Adds a listener to a node and returns an object
        // to use to remove the listener later.
        bindListener: function(nodeObj, eventName, listener) {
            var listenerFn = listener;

            if (nodeObj.addEventListener) {
                // DOM Level 2 event model
                nodeObj.addEventListener(eventName, listenerFn, false);
            } else if (nodeObj.attachEvent) {
                // IE event model
                listenerFn = function() {
                    // In IE the Event is attached to window
                    // instead of passed to the listener.
                    listener(window.event);
                }
                nodeObj.attachEvent("on" + eventName, listenerFn);
            } else {
                throw new Error("Event registration not supported");
            }

            var event = {
                nodeObj: nodeObj,
                name: eventName,
                listener: listenerFn
            };

            eventListeners.push(event);
            return event;
        },

        // Removes the given listener.
        unbindListener: function(event) {
            detachListener(event);
            deleteListener(event);
        }
    };

    helper.bindListener(window, "unload", unbindAllListeners);
    return helper;
})();

