// Create a namespace for this code.
if (typeof ajaxbook == "undefined") {
    ajaxbook = {};
}

/**
 * Has the server validate a field and updates the display
 * with a validation error message response, if any.
 */
ajaxbook.Validator = function(url) {
    var timeout = 15000;

    /** Sets the message to the server response. */
    function success(xhrRequest, field) {
        // The response is just a plain string.
        // We can take that value and directly set it
        // as the body of the span tag.
        var message = xhrRequest.responseText;
        var messageTag = document.getElementById(field.name + "Msg");

        if (message != "OK") {
            messageTag.innerHTML = message;
        } else {
            messageTag.innerHTML = "";
        }
    };

    /** Notifies the user that an error occured. */
    function failure(xhrRequest, field) {
        alert("Failed to validate " + field.name + ". Server response: "
            + xhrRequest.status + " " + xhrRequest.statusText);
    };

    /** Kicks off the validation. */
    this.validate = function(field) {
        ajaxbook.RequestHelper.sendGet(
            url + encodeURIComponent(field.value),
            success, failure, timeout, field);
    };
};

/** A validator for the year field. */
ajaxbook.yearValidator = new ajaxbook.Validator("validateYear.php?year=");

/** A validator for the zip field. */
ajaxbook.zipValidator = new ajaxbook.Validator("validateZip.php?zip=");

/** A fetcher for the car models. */
ajaxbook.CarModelFetcher = {
    url: "getCarModelsXML.php?make=",
    timeout: 15000,

    /**
     * Updates the display with the car models in the given response
     * and enables the model drop-down list with the response.
     */
    success: function(xhrRequest, selectedMake) {
        // If the returned make isn't what was selected, then error.
        var make = xhrRequest.responseXML.getElementsByTagName("make")[0];
        if (make.getAttribute("name") != selectedMake) {
            throw new Error("Returned make doesn't match selected make.");
        }

        var models = make.getElementsByTagName("model");
        var modelSelect = document.getElementById("model");
        // Remove all the options except for the "Select Make..."
        modelSelect.options.length = 1;
        // Add all the models to the model select.
        for (var i = 0; i < models.length; i++) {
            modelSelect.options[i + 1] =
                new Option(models[i].firstChild.nodeValue,
                    models[i].firstChild.nodeValue, false, false);
        }
        modelSelect.disabled = false;
    },

    /** Notifies the user that an error occured. */
    failure: function(xhrRequest, selectedMake) {
        alert("Failed to get car models. Server response: "
            + xhrRequest.status + " " + xhrRequest.statusText);
        document.getElementById("make").selectedIndex = 0;
    },

    /** Requests car models from the server for the given car make. */
    fetch: function(selectedMake) {
        ajaxbook.RequestHelper.sendGet(
            this.url + encodeURIComponent(selectedMake),
            this.success, this.failure, this.timeout, selectedMake);
    }
};

/** Adds the event handlers. */
window.onload = function() {
    var yearField = document.getElementById("year");
    yearField.onchange = function() {
        if (yearField.value.length > 0) {
            ajaxbook.yearValidator.validate(yearField);
        }
    };

    var zipField = document.getElementById("zip");
    zipField.onchange = function() {
        if (zipField.value.length > 0) {
            ajaxbook.zipValidator.validate(zipField);
        }
    };

    // Set an event handler to look up the makes
    // when a model is selected.
    var makeSelect = document.getElementById("make");
    makeSelect.onchange = function() {
        var selectedMake =
                makeSelect.options[makeSelect.selectedIndex].value;
        if ("" == selectedMake) {
            // Reset the model select when a make is deselected.
            document.getElementById("model").selectedIndex = 0;
            document.getElementById("model").disabled = true;
        } else {
            ajaxbook.CarModelFetcher.fetch(selectedMake);
        }
    };
};
