/**
 * Removes all span tags with class error.
 */
function removeErrors() {
    var spans = document.getElementsByTagName("span");
    for (var i = 0; i < spans.length; i++) {
        if (spans[i].getAttribute("class") == "error") {
            document.getElementById("carSearchForm").removeChild(spans[i]);
        }
    }
}

/**
 * Updates the display with the car models in the given response.
 */
function displayModels(selectedMake, response) {
    // If we got an error, apply it to the page.
    if (response.error) {
        var field = document.getElementById(response.error.field);
        var errorTextNode =
             document.createTextNode(response.error.message);

        // Create span tag to hold the message.
        var spanNode = document.createElement("span");
        spanNode.setAttribute("class", "error");
        spanNode.appendChild(errorTextNode);
        var form = document.getElementById("carSearchForm");
        // Insert the span tag.
        spanNode = form.insertBefore(spanNode, field.nextSibling);
    }

    // If the returned make isn't what was selected, then error.
    if (!response.make || response.make.name != selectedMake) {
        throw new Error("Returned make doesn't match selected make.");
    }

    var modelSelect = document.getElementById("model");
    // Remove all the options except for the "Select Make..."
    modelSelect.options.length = 1;
    // Add all the models to the model select.
    for (var i = 0; i < response.make.models.length; i++) {
        modelSelect.options[i + 1] =
            new Option(response.make.models[i],
                response.make.models[i], false, false);
    }
    modelSelect.disabled = false;
}

/**
 * Requests car models from the server for the given car make,
 * and enables and updates the model drop-down list with the response.
 */
function fetchModels(selectedMake) {
    // Create an XMLHttpRequest object.
    var request = RequestHelper.createXHR();

    // Define the readyState event handler to process the response.
    request.onreadystatechange = function() {
        // We only care about state 4,
        // when the response has been received.
        if (request.readyState == 4) {
            // A status of 200 tells us the response was OK.
            // A status of 304 tells us the response was pulled
            // from the browser cache.
            if (request.status == 200 || request.status == 304) {
                // Response is a JSON string.
                var jsonResponseString = request.responseText;
                removeErrors();

                // Convert the JSON string to an object.
                var response = eval("(" + jsonResponseString + ")");

                // Update the display with the data.
                displayModels(selectedMake, response);
            } else {
                // Some error occured, so notify the user.
                alert("Failed to get car models. Server response: "
                    + request.status + " " + request.statusText);
                document.getElementById("make").selectedIndex = 0;
            }
        }
    }

    var encodedYear =
            encodeURIComponent(document.getElementById("year").value);
    var encodedMake = encodeURIComponent(selectedMake);

    // Tell XMLHttpRequest what URL we want to GET.
    request.open("GET",
            "http://localhost/ajax-ch5/getCarModelsJSON.php?make="
            + encodedMake + "&year=" + encodedYear, true);
    // Send the request. Remember to pass null if not sending any data.
    request.send(null);
}

/**
 * Adds the event handlers.
 */
function init() {
    // Set an event handler to look up the makes
    // when a model is selected.
    var makeSelect = document.getElementById("make");
    makeSelect.onchange = function() {
        var selectedMake =
                makeSelect.options[makeSelect.selectedIndex].value;
        if ("" == selectedMake) {
            // Reset the model select when a make is deselected.
            document.getElementById("model").selectedIndex = 0;
            document.getElementById("model").disabled = true;
        } else {
            fetchModels(selectedMake);
        }
    };
}

// Execute init() after the page loads.
window.onload = init;