/**
 * Requests car models from the server for the given car make,
 * and enables and updates the model drop-down list with the response.
 */
function fetchModels(selectedMake) {
    // Create an XMLHttpRequest object.
    var request = RequestHelper.createXHR();

    // Define the readyState event handler to process the response.
    request.onreadystatechange = function() {
        // We only care about state 4,
        // when the response has been received.
        if (request.readyState == 4) {
            // A status of 200 tells us the response was OK.
            // A status of 304 tells us the response was pulled
            // from the browser cache.
            if (request.status == 200 || request.status == 304) {
                // The response is a string of HTML options.
                // We can take that value and directly set it
                // as the body of the model select tag.
                var models = request.responseText;
                var modelSelect = document.getElementById("model");
                modelSelect.disabled = false;
                // Parse the HTML string into DOM objects.
                var modelOptions = RequestHelper.parseHTML(models);
                // Remove all existing options from the modelSelect.
                modelSelect.options.length = 0;
                // Instead of using innerHTML we append the Nodes we created.
                modelSelect.appendChild(modelOptions);
            } else {
                // Some error occured, so notify the user.
                alert("Failed to get car models. Server response: "
                    + request.status + " " + request.statusText);
                document.getElementById("make").selectedIndex = 0;
            }
        }
    }

    // Tell XMLHttpRequest what URL we want to GET.
    request.open("GET",
            "http://localhost/ajax-ch5/getCarModels.php?make="
            + encodeURIComponent(selectedMake), true);
    // Send the request. Remember to pass null if not sending any data.
    request.send(null);
}

/**
 * Adds the event handlers.
 */
function init() {
    // Set an event handler to look up the makes
    // when a model is selected.
    var makeSelect = document.getElementById("make");
    makeSelect.onchange = function() {
        var selectedMake =
                makeSelect.options[makeSelect.selectedIndex].value;
        if ("" == selectedMake) {
            // Reset the model select when a make is deselected.
            document.getElementById("model").selectedIndex = 0;
            document.getElementById("model").disabled = true;
        } else {
            fetchModels(selectedMake);
        }
    };
}

// Execute init() after the page loads.
window.onload = init;